/*************************************************************************** 
 *
 *                       u_cmd.c Version 1.99
 *
 *      Keyboard Command Processor for DSP-10 2-meter Transciever
 *
 *
 * NOTICE: Copyright (C) 1996 to 2001 by Robert S. Larkin
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * The program author can be contacted by email at boblark@proaxis.com.
 * Alternatively contact the author at
 *     Robert S. (Bob) Larkin
 *     2982 N. W. Acacia Place
 *     Corvallis, OR  97330  USA
 *
 * For more details on the background or use of the program, pay a visit to
 * http://www.proaxis.com/~boblark/dsp10.htm  and,
 * http://www.ao.com/~gnome/
 *
 ************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <graphics.h>
#include <string.h>
#include <time.h>
#include <dos.h>
#include <bios.h>
#include <process.h>
#include "util.h"
#include "uhf_kbd.h"
#include "u_code.h"
#include "async.h"
#include "moonsun.h"
#include "lbox.h"
#include "ufilter.h"
#include "uhfa.h"

int  do_cmd(void);

int do_cmd(void)
	{
	unsigned int ichr, stat;
	long lkey;
	int irtn;

	show_last_cmd("");

	stat = _bios_keybrd(_KEYBRD_SHIFTSTATUS);
	/* Make left and right shift keys equals (bits 0 and 1) and
	 * mask out Caps Lock, Num Lock and Insert (bits 5, 6, and 7).     */
	if (stat&2)   stat |= 1;
	stat &= 0X1D;
	ichr = _bios_keybrd(_KEYBRD_READ);
	lkey = (long)ichr + (((long)stat)<<16);
	irtn=0;

	switch (lkey) {
		case MIKE_GAIN_M: {
			show_last_cmd("MIKE-");
			if (--mike_gain<0) mike_gain++;
			set_mike_gain();
			break; }
		case MIKE_GAIN_P: {
			show_last_cmd("MIKE+");
			if (++mike_gain>100) mike_gain--;
			set_mike_gain();
			break; }
		case XMT_POWER_M_L:
		case XMT_POWER_M: {
			show_last_cmd("XPWR-");
			if (--xmit_power<0) xmit_power++;
			set_xmit_power();
			break; }
		case XMT_POWER_P_L:
		case XMT_POWER_P: {
			show_last_cmd("XPWR+");
			if (++xmit_power>100) xmit_power--;
			set_xmit_power();
			break; }
		case CW_SPEED_M: {
			show_last_cmd("CW SP-");
			if (--cw_speed<0) cw_speed++;
			set_cw_speed();
			break; }
		case CW_SPEED_P: {
			show_last_cmd("CW SP+");
			if (++cw_speed>25) cw_speed--;
			set_cw_speed();
			break; }
		case T_R_TOG: {			/* Home key */
			show_last_cmd("T/R");
			if (input_if_af) break;  /* Not in audio any more */
			if (mode==EME2)
			    {
			    if (eme_state ==-1) eme_state=4; /*Ready for xmit*/
			    else
				{
				eme_state=-1;	/* Not active */
				if(transmit) t_r_toggle();
				}
			    break;
			    }
			else
			    {
			    if (ptt) break; /* No response if ptt in xmit */
			    t_r_toggle();
			    break;
			    }
			}
		case FREQ_M1: {
			show_last_cmd("FREQ-");
			if (mode != FM) tfreqL -= 1;
			new_freq();
			break; }
		case FREQ_P1: {
			show_last_cmd("FREQ+");
			if (mode != FM) tfreqL += 1;
			new_freq();
			break; }
		case FREQ_M10: {
			if (mode == FM)  tfreqL -= 5000;
			else if (kbd_alt1) tfreqL -= 100; /* Reverse 10/100 */
			else tfreqL -= 10;
			new_freq();
			show_last_cmd("FREQ-");
			break; }
		case FREQ_P10: {
			if (mode == FM)  tfreqL += 5000;
			else if (kbd_alt1) tfreqL += 100; /* Reverse 10/100 */
			else tfreqL += 10;
			new_freq();
			show_last_cmd("FREQ+");
			break; }
		case FREQ_M100: {
			if (mode == FM)  tfreqL -= 5000;
			else if (kbd_alt1) tfreqL -= 10; /* Reverse 10/100 */
			else tfreqL -= 100;
			new_freq();
			show_last_cmd("FREQ-");
			break; }
		case FREQ_P100: {
			if (mode == FM)  tfreqL += 5000;
			else if (kbd_alt1) tfreqL += 10; /* Reverse 10/100 */
			else tfreqL += 100;
			new_freq();
			show_last_cmd("FREQ+");
			break; }
		case FREQ_M1K: {
			show_last_cmd("FREQ-");
			if (mode == FM) tfreqL -= 5000;
			else tfreqL -= 1000;
			new_freq();
			break; }
		case FREQ_P1K: {
			show_last_cmd("FREQ+");
			if (mode == FM) tfreqL += 5000;
			else tfreqL += 1000;
			new_freq();
			break; }
		case FREQ_M10K: {
			show_last_cmd("FREQ-");
			tfreqL -= 10000;
			new_freq();
			break; }
		case FREQ_P10K: {
			show_last_cmd("FREQ+");
			tfreqL += 10000;
			new_freq();
			break; }
		case FREQ_M100K: {
			show_last_cmd("FREQ-");
			tfreqL -= 100000;
			new_freq();
			break; }
		case FREQ_P100K: {
			show_last_cmd("FREQ+");
			tfreqL += 100000;
			new_freq();
			break; }
		case FREQ_M1MHZ: {
			show_last_cmd("FREQ-");
			tfreqL -= 1000000;
			new_freq();
			break; }
		case FREQ_P1MHZ: {
			show_last_cmd("FREQ+");
			tfreqL += 1000000;
			new_freq();
			break; }
		case SIDETONE_LEV_M_L:
		case SIDETONE_LEV_M: {
			show_last_cmd("STONE-");
			if (--sidetone_lev<0) sidetone_lev++;
			set_cwoffset();
			break; }
		case SIDETONE_LEV_P_L:
		case SIDETONE_LEV_P: {
			show_last_cmd("STONE+");
			if (++sidetone_lev>100) sidetone_lev--;
			set_cwoffset();
			break; }
		}

	/* Be careful that any key in the following must NOT also be above */
	if (transmit==0)		/* Keys available only in receive */
	    {switch(lkey){
		case HELP: {
			show_last_cmd("Help");
			show_help();
			break;  }
		case AF_FILTER_DESIGN:  {
			show_last_cmd("Filter");
			filter_design_modalbox();
			break;  }
		case MODE_S_C:
		case MODE_S_L:  {
			show_last_cmd("MODE");
			if (input_if_af) break;
			set_mode(2);
			break; }

#if 0
>>>>>>		case NARROW_T:  {
			if(input_if_af) break;
			narrow ^= 1;
			set_mode(1);
			break; }
#endif

		case BINAURAL_T:  {
			show_last_cmd("BINRL");
			if(input_if_af) break;
			binaural ^= 1;
			set_mode(1);		/* Separate? Probably. */
			break; }

#if 0
>>>>>>		case IQ_T:  {
			if(input_if_af) break;
			iq ^= 1;
			set_mode(1);
			break; }
#endif

		case RIT_M_C:
		case RIT_M_L: {
			show_last_cmd("RIT-");
			if(rit_on) 
			    {
			    rit -= 10;
			    new_freq();
			    }
			break; }
		case RIT_P_C:
		case RIT_P_L: {
			show_last_cmd("RIT+");
			if(rit_on) 
			    {
			    rit += 10;
			    new_freq();
			    }
			break; }
		case RIT_M1_C:
		case RIT_M1_L: {
			show_last_cmd("RIT-");
			if(rit_on) 
			    {
			    rit -= 1;
			    new_freq();
			    }
			break; }
		case RIT_P1_C:
		case RIT_P1_L: {
			show_last_cmd("RIT+");
			if(rit_on) 
			    {
			    rit += 1;
			    new_freq();
			    }
			break; }
		case RIT_Z_C:
		case RIT_Z_L: {
			show_last_cmd("RIT Z");
			rit = 0;
			new_freq();
			break; }
		case RIT_O_C:
		case RIT_O_L: {
			show_last_cmd("RIT T");
			rit_toggle();
			new_freq();
			break; }
		case AF_GAIN_M: {
			set_afgain(-2);
			show_last_cmd("AF G-");
			break; }
		case AF_GAIN_P: {
			show_last_cmd("AF G+");
			set_afgain(2);
			break; }
		case AF_GAIN_M1: {
			show_last_cmd("AF G-");
			set_afgain(-1);
			break; }
		case AF_GAIN_P1: {
			show_last_cmd("AF G+");
			set_afgain(1);
			break; }
		case RF_GAIN_M: {
			if (kbd_alt3) break;
			show_last_cmd("RF G-");
			if (++rfgain>6) rfgain--;
			set_rfgain();
			break; }
		case RF_GAIN_P: {
			if (kbd_alt3) break;
			show_last_cmd("RF G+");
			if (--rfgain<0) rfgain++;
			set_rfgain();
			break; }
		case A_RF_GAIN_M: {
			if (!kbd_alt3) break;
			show_last_cmd("RF G-");
			if (++rfgain>6) rfgain--;
			set_rfgain();
			break; }
		case A_RF_GAIN_P: {
			if (!kbd_alt3) break;
			show_last_cmd("RF G+");
			if (--rfgain<0) rfgain++;
			set_rfgain();
			break; }
		case FM_SQUELCH_M_C:
		case FM_SQUELCH_M_L: {
			show_last_cmd("FM SQ-");
			if (--fm_squelch<0) fm_squelch++;
			set_fm_squelch();
			break; }
		case FM_SQUELCH_P_C:
		case FM_SQUELCH_P_L: {
			show_last_cmd("FM SQ+");
			if (++fm_squelch>99) fm_squelch--;
			set_fm_squelch();
			break; }
		case WINDOW_T_C:
		case WINDOW_T_L:  {
			if (kbd_alt2) break;
			show_last_cmd("WIN FN");
			if (++do_window>3)   do_window=0;
			set_window();
			break; }
		case A_WINDOW_T_C:
		case A_WINDOW_T_L:  {
			if (!kbd_alt2) break;
			show_last_cmd("WIN FN");
			if (++do_window>3)   do_window=0;
			set_window();
			break; }
		case FFTBW_C:
		case FFTBW_L:  {
			show_last_cmd("FT BW");
			if (mode==FM || mode==LHL7) break;
			if (++fft_bw>2)   fft_bw=0;
			set_window();
			break; }
		case AF_FILTER_T:  {
			show_last_cmd("AF FIL");
			do_af_filt ^= 1;
			set_affilt_sw();
			break; }
		case AF_FILTER_S:  {
			show_last_cmd("FIL SL");
			af_coef += 1;
			if (af_coef > 7) af_coef=0;
			set_affilt_select();
			break; }
		case FIL96:  {
			show_last_cmd("IF BW");
			/* Toggles I-Q (bit 7) & Interpolation LP filter (13)*/
			if(fft_fcn & 0X0080)   fft_fcn &= 0XDF7F;
			else fft_fcn |= 0X2080;
			set_window();
			break; }
		case DENOISE_T:  {
			show_last_cmd("LMS");
			select_denoise();
			break; }
		case DELAY_T_C:
		case DELAY_T_L:  {
			show_last_cmd("STEREO");
			set_delay_right(0);
			break; }

#if 0
		case DELAY_M: {
			if (--delay_right<1) delay_right=1;
			set_delay_right(delay_right);
			break; }
		case DELAY_P: {
			if (++delay_right>10) delay_right=10;
			set_delay_right(delay_right);
			break; }
#endif
		case NC_AVE_M:  {
			show_last_cmd("SPC AV");
			d_noncoh = -1;
			set_noncohave(1);
			break; }
		case NC_AVE_P:  {
			show_last_cmd("SPC AV");
			d_noncoh = 1;
			set_noncohave(1);
			break; }
		case RECORD_C:
		case RECORD_L: {
			show_last_cmd("RECORD");
			set_data_save();
			break; }

		/*  Replace when OK or alternate installed
		case GIF_C:
		case GIF_L: {
			show_last_cmd("GIF SAV");
			do_gif_file();
			break; }
		 */

		case DOT_TYPE:  {	/* Display dot */
			show_last_cmd("DOT TP");
			set_dot();
			break; }
		case CWOFFSET_M_L:
		case CWOFFSET_M: {
			show_last_cmd("CW TN-");
			if (cwoffset<100 || mode!=CW) break;
			cwoffset=cwoffset-10;
			set_cwoffset();
			break; }
		case CWOFFSET_P_L:
		case CWOFFSET_P: {
			show_last_cmd("CW TN+");
			if (cwoffset>=9990 || mode!=CW) break;
			cwoffset=cwoffset+10;
			set_cwoffset();
			break; }
		case DENOISE_M: {
			show_last_cmd("LMS-");
			if (do_LMS==1)
			    {
			    denoise_level--;
			    if (denoise_level<0)  denoise_level=0;
			    }
			else if (do_LMS == -1)
			    {
			    autonotch_level--;
			    if (autonotch_level<0)  autonotch_level=0;
			    }
			set_denoise();
			break; }
		case DENOISE_P: {
			show_last_cmd("LMS+");
			if (do_LMS==1)
			    {
			    denoise_level++;
			    if (denoise_level > 100)  denoise_level=100;
			    }
			else if (do_LMS == -1)
			    {
			    autonotch_level++;
			    if (autonotch_level > 100)  autonotch_level=100;
			    }
			set_denoise();
			break; }
		case AGC_DECAY_M_L:
		case AGC_DECAY_M:
			show_last_cmd("AGC-");
			if (mode==FM)  break;
			agc_decay--;
			if (agc_decay<0)
				agc_decay=0;
			set_agc(agc_decay);
			break;
		case AGC_DECAY_P_L:
		case AGC_DECAY_P:
			show_last_cmd("AGC+");
			if (mode==FM)  break;
			agc_decay++;
			if (agc_decay>10)
				agc_decay=10;
			set_agc(agc_decay);
			break;
		case BITS_8_16:		/* Toggle 8/16 bit mode from dsp */
			show_last_cmd("B 8/16");
			set_8_16(1);
			break;
		case EME_PREV:		/* Previous location for EME */
			show_last_cmd("EME L-");
			set_cur_loc(0);
			break;
		case EME_NEXT:		/* Next location for EME */
			show_last_cmd("EME L+");
			set_cur_loc(1);
			break;
		case EMEDOP_C:
		case EMEDOP_L:		/* Alt L - Toggle eme dpplr correct*/
			show_last_cmd("EME DP");
			eme_doppler = !eme_doppler;
			set_eme_doppler();
			break;

		case CODE_DATA_C:
		case CODE_DATA_L:	/* Alt A - Display code status box */
			show_last_cmd("A Box");
			if (current_info_box == &lb_eme2)  /* If box open */
			   {eme2_update(0);
			   current_info_box=0;
			   break;
			   }
			else if (mode==EME2)
			   {eme2_update(1);
			   eme2_update(2);
			   current_info_box=&lb_eme2;
			   break;
			   }
			else if (current_info_box == &lb_lhl7)  /* If open */
			   {lhl7_update(0, 0);	/* Close the box */
			   break;
			   }
			else if (mode==LHL7)
			   {lhl7_update(0, 1);
			   current_info_box=&lb_lhl7;
			   break;
			   }
			else if (current_info_box == &lb_pua43)  /* If open */
			   {pua43_update(0, 0);	/* Close the box */
			   break;
			   }
			else if (mode==PUA43)
			   {pua43_update(0, 1);
			   current_info_box=&lb_pua43;
			   break;
			   }
			if (current_info_box == &lb_lterm_int)  /* If open */
			   {lterm_int_update(0);
			   current_info_box=0;
			   break;
			   }
			else if (mode==LTI)
			   {lterm_int_update(1);
			   lterm_int_update(2);
			   current_info_box=&lb_lterm_int;
			   break;
			   }
			else if (current_info_box != &lb_freq)
			    freq_data(1, 0L, 0L, 0L, 0L, 0L, 0); /* Open */
			else
			    freq_data(0, 0L, 0L, 0L, 0L, 0L, 0); /* Close */
			break;			/* If any situation left */

		case CODE_CHANGE_C:
		case CODE_CHANGE_L:	/* Alt B - Change code parameter box */
			show_last_cmd("B Box");
			if (mode==LHL7)
			   {lhl7_modalbox();
			   break;
			   }
			else if (mode==EME2)
			   {eme2_modalbox();
			   break;
			   }
			else if (mode==PUA43)
			   {pua43_modalbox();
			   break;
			   }
			else if (mode==LTI)
			   {lterm_int_modalbox();
			   break;
			   }
			break;		/* All other modes */
		case CMDW_C:
		case CMDW_L:
			show_last_cmd("CLR LT");
			if (mode == EME2)
			   {eme2_update(-1);	/* Zero sums */
			   paveave=0.0;
			   eme_pulse_count=0;
			   break;
			   }
			else if (mode==LTI)
			   {lterm_int_update(-1);	/* Zero sums */
			   paveave=0.0;
			   eme_pulse_count=0;
			   break;
			   }
			else if (mode == PUA43)
			   {pua43_update(0, -1); /* Zero sums */
			   break;
			   }
			else
			   break;
		case CONTRAST_M: {
			show_last_cmd("CNTRST");
			if (--contrast<0) contrast++;
			set_contrast();
			break; }
		case CONTRAST_P: {
			show_last_cmd("CNTRST");
			if (++contrast>100) contrast--;
			set_contrast();
			break; }
		case BRITE_M: {
			show_last_cmd("BRITE-");
			if (--bright<0) bright++;
			set_brightness();
			break; }
		case BRITE_P: {
			show_last_cmd("BRITE+");
			if (++bright>100) bright--;
			set_brightness();
			break; }
		case DIAG_S:  {		/* Toggle Diagnostics */
			show_last_cmd("B LINE");
			diagnostics(0);
			break; }
		case NORM_QCK_C:
		case NORM_QCK_L:  {	/* Toggle Fast normalize */
			show_last_cmd("Q NORM");
			if (normalize_on)  normalize_plot(0);
			else               normalize_plot(1);
			break; }
		case NORM_PRC_C:
		case NORM_PRC_L:  {	/* Toggle Precision normalize */
			show_last_cmd("P NORM");
			if (normalize_on)  normalize_plot(0);
			else               normalize_plot(2);
			break; }
		case COLOR_AGC_C:
		case COLOR_AGC_L:  {	/* Toggle Color AGC */
			show_last_cmd("A DSPL");
			if (auto_disp_on)  do_auto_disp(0);
			else               do_auto_disp(1);
			break; }
		case DISP_T_C:
		case DISP_T_L:	{	/* Toggle waterfall display */
			if (kbd_alt2) break;
			show_last_cmd("PANEL");
			waterfall ^= 1;
			disp_toggle();
			break; }
		case A_DISP_T_C:
		case A_DISP_T_L:{	/* Toggle waterfall display */
			if (!kbd_alt2) break;
			show_last_cmd("PANEL");
			waterfall ^= 1;
			disp_toggle();
			break; }
		case OFFSET_M: {
			show_last_cmd("OFFST-");
			koffset -= 1.0;
			if (koffset<-50.0) koffset=-50.0;
			set_disp_offset(1);
			break; }
		case OFFSET_P: {
			show_last_cmd("OFFST+");
			koffset += 1.0;
			if (koffset>50.0) koffset=50.0;
			set_disp_offset(1);
			break; }
		case DB_DIV: {
			show_last_cmd("DB/DIV");
			db_div_set += 1;
			if (db_div_set>3)  db_div_set=0;
			set_db_div(1);
			break; }
		case DO_QUIT: {		/* Exit to DOS */
			show_last_cmd("EXIT");
			irtn=1;
			break; }
		case WARM_BOOT: {	/* Just in case...  */
			show_last_cmd("DSP WB");
			do_warm_boot();
			break; }
		case INPUT_S_C:
		case INPUT_S_L: {
			if (kbd_alt2) break;
			show_last_cmd("RF/AUD");
			input_if_af ^= 1; /* Toggle input */
			set_input();
			break; }
		case A_INPUT_S_C:
		case A_INPUT_S_L: {
			if (!kbd_alt2) break;
			show_last_cmd("RF/AUD");
			input_if_af ^= 1; /* Toggle input */
			set_input();
			break; }
		case SET_CLOCK_C:
		case SET_CLOCK_L: {
			show_last_cmd("CLOCK");
			set_clock(1);
			break; }
		case CMDQ_C:
		case CMDQ_L:  {
			show_last_cmd("LTFILE");
			save_lt_pwr();
			break; }
		case CMDE_C:
		case CMDE_L:  {
			show_last_cmd("FLATTN");
			lt_normalize();
			break; }
		case K_SF9:   {
			show_last_cmd("LMSDCY");
			denoise_decay -= 1;
			if (denoise_decay<0)  denoise_decay=0;
			set_denoise();
			break; }
		case K_SF10:   {
			show_last_cmd("LMSDCY");
			denoise_decay += 1;
			if (denoise_decay > 99)  denoise_decay=99;
			set_denoise();
			break; }

#if 0
		case K_SF9:   {
			denoise_del_len--;
			if (denoise_del_len<0) denoise_del_len++;
			set_denoise();
			break; }
		case K_SF10:   {
			denoise_del_len++;
			if (denoise_del_len>1) denoise_del_len--;
			set_denoise();
			break; }
#endif
		case BAND_50_A:   show_last_cmd("TVTR A");new_band(0);break;
		case BAND_144_B:  show_last_cmd("TVTR B");new_band(1);break;
		case BAND_220_C:  show_last_cmd("TVTR C");new_band(2);break;
		case BAND_432_D:  show_last_cmd("TVTR D");new_band(3);break;
		case BAND_902_9:  show_last_cmd("TVTR 9");new_band(4);break;
		case BAND_1296_E: show_last_cmd("TVTR E");new_band(5);break;
		case BAND_2304_F: show_last_cmd("TVTR F");new_band(6);break;
		case BAND_3456_G: show_last_cmd("TVTR G");new_band(7);break;
		case BAND_5760_H: show_last_cmd("TVTR H");new_band(8);break;
		case BAND_10K_I:  show_last_cmd("TVTR I");new_band(9);break;
		case BAND_24K_J:  show_last_cmd("TVTR J");new_band(10);break;
		case BAND_47K_K:  show_last_cmd("TVTR K");new_band(11);break;
		case BAND_75K_L:  show_last_cmd("TVTR L");new_band(12);break;
		case BAND_119K_M: show_last_cmd("TVTR M");new_band(13);break;
		case BAND_145K_N: show_last_cmd("TVTR N");new_band(14);break;
		case BAND_241K_O: show_last_cmd("TVTR O");new_band(15);break;
		case BAND_MEMO_1: show_last_cmd("VFO 1");new_band(16);break;
		case BAND_MEMO_2: show_last_cmd("VFO 2");new_band(17);break;
		case BAND_MEMO_3: show_last_cmd("VFO 3");new_band(18);break;
		case BAND_MEMO_4: show_last_cmd("VFO 4");new_band(19);break;
		case BAND_MEMO_5: show_last_cmd("VFO 5");new_band(20);break;
		case CW_BUF_CLR: {
			show_last_cmd("CLR BF");
			morse(INIT_CW, 0);
			if (mode==CW || mode==LTI)
			    print_medium(0, -1);
			else if (mode==LHL7 || mode==PUA43)
			    print_medium_1(0, -1);
			break; }
		default: break;
		}
	    }
	if (mode==CW || mode==LHL7 || mode==LTI)
	   {
	   ichr=ichr & 0XFF;
	   /* Not scrl, ctrl, alt, but either alphnumeric or Back Space */
	   if ((!(lkey & 0X1C0000)) &&
		 ((ichr>=32 && ichr<127 && ichr!='<' && ichr!='>')
			 || ichr==8))
		{
		morse(ADD_CH, ichr);
		sprintf(cc, "'%c'", ichr);
		show_last_cmd(cc);
		}
	   }
	return(irtn);
	}
